VERSION 5.00
Begin VB.UserControl NumericTextBox 
   ClientHeight    =   336
   ClientLeft      =   0
   ClientTop       =   0
   ClientWidth     =   4800
   ScaleHeight     =   336
   ScaleWidth      =   4800
   Begin VB.TextBox txtValue 
      Height          =   330
      Left            =   0
      TabIndex        =   0
      Text            =   "0"
      Top             =   0
      Width           =   1710
   End
End
Attribute VB_Name = "NumericTextBox"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Option Explicit
Enum BadThing
  OutOfRange = 1
  NotNumeric
End Enum

Public Event BadValue(WhatsWrong As Integer)
Public Event BadKey()
'Default  Values for the property
Const mOriginalValue = 0
Const mOriginalMinValue = -1000
Const mOriginalMaxValue = 1000
Const mOriginalRange = 1000
'Private Instance Field Variables:
Dim mValue As Double
Dim mMinValue As Double
Dim mMaxValue As Double
Dim mRange As Variant 'to allow for no range at all


Private Sub txtValue_Change()
  'check the current value for being a number
  If Not IsNumeric(txtValue.Text) Then
    RaiseEvent BadValue(NotNumeric)
    txtValue.Text = mValue
    'check the current value for proper range
  ElseIf Val(txtValue.Text) < mMinValue Then
    'raise the BadValue event
    RaiseEvent BadValue(OutOfRange)
    If OutOfRange Then
      'reset to the min value
      txtValue.Text = mMinValue
    Else
      'reset to previous value
      txtValue.Text = mValue
    End If
  ElseIf Val(txtValue.Text) > mMaxValue Then
    'raise the event
    RaiseEvent BadValue(OutOfRange)
    If OutOfRange Then
      'reset to the max value
      txtValue.Text = mMaxValue
    Else
      'reset to previous value
      txtValue.Text = mValue
    End If
  End If
  'set the instance variable directly
  mValue = Val(txtValue.Text)
End Sub

Private Sub txtValue_KeyPress(KeyAscii As Integer)
  Static bDecimalPointUsedUp As Boolean
  Static bMinusSignAlready As Boolean
  Select Case KeyAscii
  Case vbKeyBack, vbKeyRight, vbKeyLeft
    'do nothing
  Case Asc("0") To Asc("9")
    ' do nothing
  Case Asc("-")
    If txtValue.SelStart <> 0 Or bMinusSignAlready Then
      KeyAscii = 0
      RaiseEvent BadKey
    Else
      bMinusSignAlready = True
    End If
  Case Asc(".")
    If bDecimalPointUsedUp Then
      KeyAscii = 0
      RaiseEvent BadKey
    Else
      bDecimalPointUsedUp = True
    End If
  Case Else
    KeyAscii = 0
    RaiseEvent BadKey
  End Select
End Sub


Private Sub txtValue_LostFocus()
  If Not (IsNumeric(txtValue.Text)) Then
    RaiseEvent BadValue(NotNumeric)
  End If
End Sub

Private Sub UserControl_Resize()
  'stretch to width and height of control
  txtValue.Width = Width
  txtValue.Height = Height
End Sub

Public Property Get Value() As Double
  Value = mValue
End Property

Public Property Let Value(ByVal New_Value As Double)
  If New_Value < mMinValue Then
    MsgBox "Minimum Limit Exceeded!"
    mValue = mMinValue
  ElseIf New_Value > mMaxValue Then
    MsgBox "Maximum Limit Exceeded!"
    mValue = mMaxValue
  Else
    mValue = New_Value
  End If
  txtValue.Text = mValue
  PropertyChanged "Value"
End Property

Public Property Get MinValue() As Double
  MinValue = mMinValue
End Property

Public Property Let MinValue(ByVal New_MinValue As Double)
  If New_MinValue > mMaxValue Then
    MsgBox "Minimum cannot exceed Maximum!"
  Else
    mMinValue = New_MinValue
    PropertyChanged "MinValue"
  End If
End Property

Public Property Get MaxValue() As Double
  MaxValue = mMaxValue
End Property

Public Property Let MaxValue(ByVal New_MaxValue As Double)
  If New_MaxValue < mMinValue Then
    MsgBox "Maximum cannot be less than Minimum!"
  Else
    mMaxValue = New_MaxValue
    PropertyChanged "MaxValue"
  End If
  PropertyChanged "Range"
End Property

Public Property Get Range() As Variant
  Range = mRange
End Property


'Initialize Properties
Private Sub UserControl_InitProperties()
  mValue = mOriginalValue
  mMinValue = mOriginalMinValue
  mMaxValue = mOriginalMaxValue
  mRange = mOriginalRange
End Sub

'Load property values from storage
Private Sub UserControl_ReadProperties(PropBag As PropertyBag)
  On Error Resume Next
  mMinValue = PropBag.ReadProperty("MinValue", mOriginalMinValue)
  mMaxValue = PropBag.ReadProperty("MaxValue", mOriginalMaxValue)
  Value = PropBag.ReadProperty("Value", mOriginalValue)
End Sub

'Write property values to storage
Private Sub UserControl_WriteProperties(PropBag As PropertyBag)
  On Error Resume Next
  Call PropBag.WriteProperty("MinValue", mMinValue, _ 
mOriginalMinValue)
  Call PropBag.WriteProperty("MaxValue", mMaxValue, _ 
mOriginalMaxValue)
  Call PropBag.WriteProperty("Value", mValue, mOriginalValue)
End Sub

